
/*
kcmkwix.cpp - implementation file for the KwiX kcontrol module.

Terms and Conditions

Copyright (c) 2001 Catalin Climov <xxl@thekompany.com>

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
COPYRIGHT HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the copyright holder shall not
be used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from the copyright holder.
*/

/*
 * KwiX setup module.
 */

#include "kcmkwix.h"
#include "ksubsetselector.h"
#include "btnhighcolor.h"

#include <kglobal.h>
#include <klocale.h>
#include <qlayout.h>
#include <qbitmap.h>
#include <kconfig.h>
#include <dcopclient.h>
#include <kapp.h>

//extern "C"
//{
//    int __kde_do_not_unload = 1;
//}

extern "C"
{
    KCModule *create_kwix(QWidget *parent, char *name)
    {
        KGlobal::locale()->insertCatalogue("kcmkwix");
        return new KwixCM(parent, name);
    }

    void init_kwix()
    {
    }
}

static const char * title_xpm[]={
"14 15 6 1",
". c None",
"# c #000000",
"d c #000080",
"b c #0058c0",
"a c #0080ff",
"c c #58a8ff",
"..............",
"##############",
"aaaaaaaaaaaaaa",
"bbbbbbbbbbbbbb",
"bbbbbccbbbbbbc",
"bccbbcdbbccbbc",
"bcdbbbbbbcdbbb",
"bbbbbbbbbbbbbb",
"bbbbbccbbbbbbc",
"bccbbcdbbccbbc",
"bcdbbbbbbcdbbb",
"bbbbbbbbbbbbbb",
"dddddddddddddd",
"##############",
".............."};

static unsigned char iconify_bits[] = {
  0x00, 0x00, 0xff, 0xff, 0x7e, 0x3c, 0x18, 0x00};

/* not used currently */
static unsigned char close_bits[] = {
    0xc3, 0x66, 0x3c, 0x18, 0x3c, 0x66, 0xc3, 0x00};

static unsigned char maximize_bits[] = {
  0x00, 0x18, 0x3c, 0x7e, 0xff, 0xff, 0x00, 0x00};

static unsigned char sticky_bits[] = {
   0x3c, 0x42, 0x81, 0x81, 0x81, 0x81, 0x42, 0x3c};

static unsigned char question_bits[] = {
    0x3c, 0x66, 0x60, 0x30, 0x18, 0x00, 0x18, 0x18};

KwixCM::KwixCM(QWidget *parent, const char *name)
    : KCModule(parent, name), _selected(14)
{
    QGridLayout* layout = new QGridLayout(this);
    _selector = new KSubsetSelector(this);
    layout->addWidget(_selector,0,0);
    connect(_selector,SIGNAL(selectionModified()),SLOT(slotSelectionModified()));
    createDefaults();
    setButtons(KCModule::Help | KCModule::Default | KCModule::Apply);
    load();
}

KwixCM::~KwixCM()
{
}

void KwixCM::load()
{
    KConfig cfg("kwixrc");
    if( cfg.hasGroup("Buttons") )
    {
        cfg.setGroup("Buttons");
        if( cfg.hasKey("Layout") )
        {
            QStringList strl = cfg.readListEntry("Layout");
            QArray<int> tmp(strl.count());
            for( int i=0; i < (int)strl.count(); i++)
                if( strl[i] == "Separator" )
                    tmp[i] = Separator_ID;
                else if( strl[i] == "Menu" )
                    tmp[i] = Menu_ID;
                else if( strl[i] == "Title" )
                    tmp[i] = Title_ID;
                else if( strl[i] == "Help" )
                    tmp[i] = Help_ID;
                else if( strl[i] == "Sticky" )
                    tmp[i] = Sticky_ID;
                else if( strl[i] == "Minimize" )
                    tmp[i] = Minimize_ID;
                else if( strl[i] == "Maximize" )
                    tmp[i] = Maximize_ID;
                else if( strl[i] == "Close" )
                    tmp[i] = Close_ID;
                else
                    tmp[i] = Separator_ID;
            _saved = tmp;
        }
        else
            _saved = _selected;
    }
    else
        _saved = _selected;
    _selector->setElements(_default,_saved);
}

void KwixCM::save()
{
    _saved = _selector->selectedElements();
    KConfig cfg("kwixrc");
    cfg.setGroup("Buttons");
    QStringList strl;
    for( int i = 0; i < (int)_saved.count(); i++ )
        switch( _saved[i] )
        {
            case Separator_ID:
                strl.append("Separator");
                break;
            case Menu_ID:
                strl.append("Menu");
                break;
            case Title_ID:
                strl.append("Title");
                break;
            case Help_ID:
                strl.append("Help");
                break;
            case Sticky_ID:
                strl.append("Sticky");
                break;
            case Minimize_ID:
                strl.append("Minimize");
                break;
            case Maximize_ID:
                strl.append("Maximize");
                break;
            case Close_ID:
                strl.append("Close");
                break;
        }
    cfg.writeEntry("Layout",strl);
    // reconfigure kwin (Waldo Bastian)
    DCOPClient *client = kapp->dcopClient();
    if (!client->isAttached())
       client->attach();
    client->send("kwin", "", "reconfigure()", "");
}

void KwixCM::defaults()
{
    _selector->setElements(_default,_selected);
}

void KwixCM::createDefaults()
{
    QPixmap base(btnhighcolor_xpm);

    QBitmap close_b(8,8,close_bits,true);
    QPixmap close(base);
    bitBlt(&close,3,4,&close_b,0,0,8,8,Qt::AndROP);

    QBitmap help_b(8,8,question_bits,true);
    QPixmap help(base);
    bitBlt(&help,3,4,&help_b,0,0,8,8,Qt::AndROP);

    QBitmap sticky_b(8,8,sticky_bits,true);
    QPixmap sticky(base);
    bitBlt(&sticky,3,4,&sticky_b,0,0,8,8,Qt::AndROP);

    QBitmap maximize_b(8,8,maximize_bits,true);
    QPixmap maximize(base);
    bitBlt(&maximize,3,4,&maximize_b,0,0,8,8,Qt::AndROP);

    QBitmap minimize_b(8,8,iconify_bits,true);
    QPixmap minimize(base);
    bitBlt(&minimize,3,4,&minimize_b,0,0,8,8,Qt::AndROP);

    QBitmap separator(14,15,true);
    QPixmap title(title_xpm);

    _default.append(new KSubsetElement(Separator_ID,separator,"-----",true));
    _default.append(new KSubsetElement(Menu_ID,base,"Menu"));
    _default.append(new KSubsetElement(Title_ID,title,"Title",false,true));
    _default.append(new KSubsetElement(Help_ID,help,"Help"));
    _default.append(new KSubsetElement(Sticky_ID,sticky,"Sticky"));
    _default.append(new KSubsetElement(Minimize_ID,minimize,"Minimize"));
    _default.append(new KSubsetElement(Maximize_ID,maximize,"Maximize"));
    _default.append(new KSubsetElement(Close_ID,close,"Close"));
    _selected[ 0]=Separator_ID;
    _selected[ 1]=Separator_ID;
    _selected[ 2]=Separator_ID;
    _selected[ 3]=Close_ID;
    _selected[ 4]=Separator_ID;
    _selected[ 5]=Separator_ID;
    _selected[ 6]=Title_ID;
    _selected[ 7]=Separator_ID;
    _selected[ 8]=Separator_ID;
    _selected[ 9]=Help_ID;
    _selected[10]=Sticky_ID;
    _selected[11]=Minimize_ID;
    _selected[12]=Maximize_ID;
    _selected[13]=Separator_ID;
}

QString KwixCM::quickHelp() const
{
    return i18n("<h1>Configure the KwiX window decoration</h1>"
      " Only the button layout is configurable so far.<p>"
    );
}

void KwixCM::slotSelectionModified()
{
    emit changed(true);
}

#include "kcmkwix.moc"
