
/*
ksubsetselector.h - header file for the KSubSetSelector class.

Terms and Conditions

Copyright (c) 2001 Catalin Climov <xxl@thekompany.com>

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
COPYRIGHT HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the copyright holder shall not
be used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from the copyright holder.
*/

#ifndef KSUBSETSELECTOR_H
#define KSUBSETSELECTOR_H

#include <qwidget.h>
#include <qpixmap.h>
#include <qlistbox.h>
#include <qlist.h>
#include <qarray.h>
class KListBox;
class QToolButton;

class KSubsetElement : public QListBoxPixmap
{
public:
    KSubsetElement(const int id, const QPixmap& pixmap, const QString& text, const bool multi = false, const bool alwaysSelected = false)
        :QListBoxPixmap(pixmap, text)
    {
        _id = id;
        _multi = multi;
        _alwaysSelected = alwaysSelected;
    };
    KSubsetElement(const KSubsetElement& e )
        :QListBoxPixmap(*(e.pixmap()), e.text())
    {
        _id = e._id;
        _multi = e._multi;
        _alwaysSelected = e._alwaysSelected;
    }
    KSubsetElement(const KSubsetElement* e )
        :QListBoxPixmap(*(e->pixmap()), e->text())
    {
        _id = e->_id;
        _multi = e->_multi;
        _alwaysSelected = e->_alwaysSelected;
    }
    virtual ~KSubsetElement() {};
    virtual const int id() { return _id; };
    virtual const bool multi() { return _multi; };
    virtual const bool alwaysSelected() { return _alwaysSelected; };

private:
    int _id;
    bool _multi;
    bool _alwaysSelected;
};

typedef QList<KSubsetElement> KSubset;

class KSubsetSelector : public QWidget
{
    Q_OBJECT

public:
    KSubsetSelector( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~KSubsetSelector();

    void setElements( const KSubset& full, const QArray<int>& selected );
    const QArray<int> selectedElements();

signals:
    void selectionModified();
    void orderChanged();
    void selectionChanged();

private slots:
    void slotLeftSelectionChanged();
    void slotRightSelectionChanged();
    void slotRightArrowClicked();
    void slotLeftArrowClicked();
    void slotUpArrowClicked();
    void slotDownArrowClicked();
    void slotLeftHighlighted(int);
    void slotRightHighlighted(int);
private:
    void setupUI();
    void setupConnections();
    QToolButton* _leftArrow;
    QToolButton* _upArrow;
    QToolButton* _rightArrow;
    QToolButton* _downArrow;
    KListBox* _leftBox;
    KListBox* _rightBox;
    KSubset _full;
    QArray<int> _selected;
};

#endif // KSUBSETSELECTOR_H
