
/*
kwix.cpp - implementation file for the "KwiX" kwin theme.Terms and Conditions
 
Copyright (c) 2001 Catalin Climov <xxl@thekompany.com>
Parts Copyright (c) 2000-2001 Daniel M. Duley <mosfet@kde.org>
Based on the ModSystem kwin theme.
 
Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.
 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
COPYRIGHT HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
Except as contained in this notice, the name of the copyright holder shall not
be used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from the copyright holder.
*/

#include <kconfig.h>
#include "kwix.h"
#include <qapplication.h>
#include <qcursor.h>
#include <qabstractlayout.h>
#include <qlayout.h>
#include <qtoolbutton.h>
#include <qlabel.h>
#include <qdrawutil.h>
#include <kpixmapeffect.h>
#include <kdrawutil.h>
#include <kapp.h>
#include <qbitmap.h>
//#include "../../workspace.h"
#include <kwin/workspace.h>
#include <kwin/options.h>

#include "buttondata.h"
#include "btnhighcolor.h"
#include <kglobal.h>
#include <klocale.h>

using namespace KWinInternal;

extern "C"
{
    Client *allocate(Workspace *ws, WId w, int)
    {
        return(new Kwix(ws, w));
    }
}


static unsigned char iconify_bits[] = {
 0x00,0x00,0x00,0xfe,0xfe,0x7c,0x38,0x10,0x00,0x00,0x00,0x48,0x00,0x00,0x00,
 0x89,0x00,0x00,0x00,0x00,0x24,0x00,0x6b,0x00,0x69,0x00,0x63,0x00,0x6f,0x00,
 0x5f,0x00 };
//  0x00, 0x00, 0xff, 0xff, 0x7e, 0x3c, 0x18, 0x00};

static unsigned char close_bits[] = {
 // 0x86,0x01,0xce,0x2f,0xfc,0xf8,0x78,0xa8,0x78,0x00,0xfc,0xae,0xce,0xf9,0x86,
 //0x53,0x00,0x00,0x00,0x00,0x50,0x00,0x00,0x00,0x11,0x00,0x00,0x00,0x30,0x68,
 //0x0b,0x08,0x20,0x58,0x0b,0x08 };
 //0xc3,0xe7,0x7e,0x3c,0x3c,0x7e,0xe7,0xc3,0x00,0x00,0x00,0x48,0x00,0x00,0x00,
 //0x21,0x00,0x00,0x00,0x00,0x75,0x00,0x6e,0x00,0x6b,0x00,0x6e,0x00,0x6f,0x00,
 //0x77,0x00 };
  0xc6,0xee,0x7c,0x38,0x7c,0xee,0xc6,0x00,0x00,0x00,0x00,0x48,0x00,0x00,0x00,
 0x11,0x00,0x00,0x00,0x24,0xbc,0x10,0x08,0x00,0x00,0x00,0x00,0x24,0x3e,0x1a,
 0x40,0xb1 };

static unsigned char maximize_bits[] = {
 0x10,0x38,0x7c,0xfe,0xfe,0x00,0x00,0x00,0x00,0x00,0x00,0x48,0x00,0x00,0x00,
 0x29,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x10,0x00,0x00,
 0x00,0x01 };
//  0x00, 0x18, 0x3c, 0x7e, 0xff, 0xff, 0x00, 0x00};

static unsigned char minmax_bits[] = {
 0x0c,0x1c,0x3c,0x7c,0xfc,0xfc,0x00,0x00,0x00,0x00,0x00,0x48,0x00,0x00,0x00,
 0x11,0x00,0x00,0x00,0x70,0xfc,0x0c,0x08,0x00,0x00,0x00,0x00,0xe1,0xe5,0x08,
 0x40,0x31 };
//    0x0c, 0x18, 0x33, 0x67, 0xcf, 0x9f, 0x3f, 0x3f};

static unsigned char unsticky_bits[] = {
 0x00,0x00,0x7e,0x3c,0x3c,0x3c,0x7e,0x00,0x00,0x00,0x00,0x48,0x00,0x00,0x00,
 0x19,0x00,0x00,0x00,0x68,0x13,0x10,0x08,0x00,0x00,0xff,0xff,0x10,0x61,0x17,
 0x40,0x7a };

static unsigned char sticky_bits[] = {
 //0x00,0x00,0x20,0x99,0xe0,0xf9,0xff,0xa9,0xfe,0x01,0xe0,0x0b,0x20,0xf9,0x00,
 //0x52,0x00,0x00,0x00,0x00,0x50,0x00,0x00,0x00,0x79,0x00,0x00,0x00,0x00,0x00,
 //0x00,0x00,0xf7,0x00,0x40,0x01 };
 0x00,0x90,0xf0,0xff,0xfe,0xf0,0x90,0x00,0x00,0x00,0x00,0x48,0x00,0x00,0x00,
 0x29,0x00,0x00,0x00,0x00,0x52,0xc5,0x40,0x40,0xf8,0x99,0x40,0x00,0x00,0x00,
 0x00,0x00 };

static unsigned char question_bits[] = {
    0x3c, 0x66, 0x60, 0x30, 0x18, 0x00, 0x18, 0x18};

static unsigned char btnhighcolor_mask_bits[] = {
0xe0,0x03,0xf8,0x0f,0xfc,0x9f,0xfe,0xbf,0xfe,0x3f,0xff,0x7f,0xff,0xff,0xff,
 0xff,0xff,0x7f,0xff,0x7f,0xfe,0xbf,0xfe,0x3f,0xfc,0x1f,0xf8,0x0f,0xe0,0x03,
 0x00,0x40,0x80,0x00,0x00,0x00,0x19,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x08,
 0x00,0x00,0x00,0xe8,0x67,0x0e,0x08,0x10,0x72,0x0f,0x08,0x30,0xb1,0x10,0x08 };


static KPixmap *aUpperGradient=0;
static KPixmap *iUpperGradient=0;
static QPixmap *buttonPix=0;
static QPixmap *buttonPixDown=0;
static QPixmap *iButtonPix=0;
static QPixmap *iButtonPixDown=0;

static QColor buttonFg;
static bool pixmaps_created = false;

static void make_button_fx(const QColorGroup &g, QPixmap *pix, bool light=false)
{
    static QBitmap lcDark1(15, 15, lowcolor_6a696a_bits, true);
    static QBitmap lcDark2(15, 15, lowcolor_949194_bits, true);
    static QBitmap lcDark3(15, 15, lowcolor_b4b6b4_bits, true);
    static QBitmap lcLight1(15, 15, lowcolor_e6e6e6_bits, true);
    static QImage btnSource(btnhighcolor_xpm);

    pix->fill(g.background());
    QPainter p(pix);

    if(QPixmap::defaultDepth() > 8){
        int i, destH, destS, destV, srcH, srcS, srcV;
        QColor btnColor = g.background();

        if(btnSource.depth() < 32)
            btnSource = btnSource.convertDepth(32);
        if(light)
            btnColor = btnColor.light(120);
        btnColor.hsv(&destH, &destS, &destV);
        QImage btnDest(15, 15, 32);

        unsigned int *srcData = (unsigned int *)btnSource.bits();
        unsigned int *destData = (unsigned int *)btnDest.bits();
        QColor srcColor;
        for(i=0; i < btnSource.width()*btnSource.height(); ++i){
            srcColor.setRgb(srcData[i]);
            srcColor.hsv(&srcH, &srcS, &srcV);
            srcColor.setHsv(destH, destS, srcV);
            destData[i] = srcColor.rgb();
        }
        pix->convertFromImage(btnDest);

    }
    else{
        if(!lcDark1.mask()){
            lcDark1.setMask(lcDark1);
            lcDark2.setMask(lcDark2);
            lcDark3.setMask(lcDark3);
            lcLight1.setMask(lcLight1);
        }
        p.setPen(g.dark());
        p.drawPixmap(0, 0, lcDark2);
        p.drawPixmap(0, 0, lcDark1);
        p.setPen(g.mid());
        p.drawPixmap(0, 0, lcDark3);
        p.setPen(g.light());
        p.drawPixmap(0, 0, lcLight1);
    }
}


static void create_pixmaps()
{
    if(pixmaps_created)
        return;
    pixmaps_created = true;

    if(QPixmap::defaultDepth() > 8){
        aUpperGradient = new KPixmap;
        aUpperGradient->resize(32, 18);
        iUpperGradient = new KPixmap;
        iUpperGradient->resize(32, 18);
        KPixmapEffect::gradient(*aUpperGradient,
                                options->color(Options::TitleBar, true).light(130),
                                options->color(Options::TitleBlend, true),
                                KPixmapEffect::VerticalGradient);
        KPixmapEffect::gradient(*iUpperGradient,
                                options->color(Options::TitleBar, false).light(130),
                                options->color(Options::TitleBlend, false),
                                KPixmapEffect::VerticalGradient);
    }
    // buttons
    QColorGroup btnColor(options->colorGroup(Options::ButtonBg, true));
    buttonPix = new QPixmap(15, 15);
    make_button_fx(btnColor, buttonPix);
    buttonPixDown = new QPixmap(15, 15);
    make_button_fx(btnColor, buttonPixDown, true);

    btnColor = options->colorGroup(Options::ButtonBg, false);
    iButtonPix = new QPixmap(15, 15);
    make_button_fx(btnColor, iButtonPix);
    iButtonPixDown = new QPixmap(15, 15);
    make_button_fx(btnColor, iButtonPixDown, true);


    if(qGray(btnColor.background().rgb()) < 150)
        buttonFg = Qt::white;
    else
        buttonFg = Qt::black;
}

ModernButton::ModernButton(Client *parent, const char *name,
                           const unsigned char *bitmap)
    : QToolButton(parent, name)
{
    QBitmap mask(15, 15, QPixmap::defaultDepth() > 8 ?
                 btnhighcolor_mask_bits : lowcolor_mask_bits, true);
    resize(15, 15);
    client = parent;
    if(!strcmp(this->name(),"menu") ){//&& !client->miniIcon().isNull()){     
    }else{
      if(bitmap)
        setBitmap(bitmap);
      setMask(mask);
    }
    setBackgroundColor(options->color( Options::TitleBar,true) );
}

QSize ModernButton::sizeHint() const
{
    return(QSize(15, 15));
}

void ModernButton::reset()
{
    repaint(false);
}

void ModernButton::setBitmap(const unsigned char *bitmap)
{
    // here comes the size of the button-symbols!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    deco = QBitmap(8, 8, bitmap, true);
    deco.setMask(deco);
    repaint();
}

void ModernButton::drawButton(QPainter *p)
{
  if(!strcmp(this->name(),"menu")){// && !client->miniIcon().isNull()){
    QPixmap px;
    //px.fill(options->color(Options::TitleBar, true));
    // px.copy(client->miniIcon())
      //p->drawPixmap(0,0,px);
//      if(client->isActive()){
//        setBackgroundColor(options->color( Options::TitleBar,true) );
//      }else{
//        setBackgroundColor(options->color( Options::TitleBlend,true) );
//      }
    p->drawPixmap(0,0,client->miniIcon());
  }else{
    if(client->isActive()){
        if(buttonPix)
            p->drawPixmap(0, 0, isDown() ? *buttonPixDown : *buttonPix);
    }
    else{
        if(iButtonPix)
            p->drawPixmap(0, 0, isDown() ? *iButtonPixDown : *iButtonPix);
    }
    if(!deco.isNull()){
        p->setPen(buttonFg);
        p->drawPixmap(isDown() ? 4 : 3, isDown() ? 5 : 4, deco);
    }
  }


}

void ModernButton::mousePressEvent( QMouseEvent* e )
{
    last_button = e->button();
    QMouseEvent me ( e->type(), e->pos(), e->globalPos(), LeftButton, e->state() );
    QButton::mousePressEvent( &me );
}

void ModernButton::mouseReleaseEvent( QMouseEvent* e )
{
    QMouseEvent me ( e->type(), e->pos(), e->globalPos(), LeftButton, e->state() );
    QButton::mouseReleaseEvent( &me );
}


void Kwix::slotReset()
{
    if(aUpperGradient){
        delete aUpperGradient;
        delete iUpperGradient;
    }
    delete buttonPix;
    delete buttonPixDown;
    delete iButtonPix;
    delete iButtonPixDown;

    pixmaps_created = false;
    create_pixmaps();
    for( int i = 0; i < 6; i++ )
            delete button[i];
    delete titlebar;
    createButtons(hb);
    titleBuffer.resize(0, 0);
    recalcTitleBuffer();
    repaint();
}

Kwix::Kwix( Workspace *ws, WId w, QWidget *parent,
                            const char *name )
    : Client( ws, w, parent, name, WResizeNoErase ), titleLayout(14)
{
    create_pixmaps();
    connect(options, SIGNAL(resetClients()), this, SLOT(slotReset()));

    QGridLayout* g = new QGridLayout(this, 0, 0, 2);
    g->addWidget(windowWrapper(), 1, 1 );
    g->setRowStretch(1, 10);
    g->addItem( new QSpacerItem( 0, 0, QSizePolicy::Fixed, QSizePolicy::Expanding ) );

    g->addColSpacing(0, 2);
//    g->addColSpacing(2, 8);
    g->addColSpacing(2, 3);

//    g->addRowSpacing(2, 8);
    g->addRowSpacing(2, 3);

    hb = new QHBoxLayout(0);
    hb->setResizeMode(QLayout::FreeResize);
    g->addLayout( hb, 0, 1 );
    createButtons(hb);

    setBackgroundMode(NoBackground);
    recalcTitleBuffer();

}

void Kwix::maxButtonClicked( )
{
    switch ( button[3]->last_button ) {
    case MidButton:
       maximize( MaximizeVertical );
       break;
    case RightButton:
       maximize( MaximizeHorizontal );
       break;
    default: //LeftButton:
       maximize( MaximizeFull );
       break;
    }
}



void Kwix::resizeEvent( QResizeEvent* )
{
    //Client::resizeEvent( e );
    recalcTitleBuffer();
    doShape();
    /*
    if ( isVisibleToTLW() && !testWFlags( WNorthWestGravity )) {
        QPainter p( this );
	QRect t = titlebar->geometry();
	t.setTop( 0 );
	QRegion r = rect();
	r = r.subtract( t );
	p.setClipRegion( r );
	p.eraseRect( rect() );
        }*/
}

void Kwix::recalcTitleBuffer()
{
    if(oldTitle == caption() && width() == titleBuffer.width())
        return;
    QFontMetrics fm(options->font(true));
    titleBuffer.resize(width(), 18);
    QPainter p;
    p.begin(&titleBuffer);
//    if(aUpperGradient)
//        p.drawTiledPixmap(0, 0, width(), 18, *aUpperGradient);
//    else
//        p.fillRect(0, 0, width(), 18,
//                   options->colorGroup(Options::TitleBar, true).
//                   brush(QColorGroup::Button));

    QRect t = titlebar->geometry();
    t.setTop( 2 );
    t.setLeft( t.left() + 4 );
    t.setRight( t.right() - 2 );

    QRegion r(t.x(), 0, t.width(), 18);
    r -= QRect(t.x()+((t.width()-fm.width(caption()))/2)-4,
               0, fm.width(caption())+8, 18);

//    p.fillRect(0, 0, t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4, 18,
    p.fillRect(0, 0, t.right(), 18,
               options->colorGroup(Options::TitleBar, true).
               brush(QColorGroup::Button));
    if(iUpperGradient)
        p.drawTiledPixmap(t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4, 0, width(), 18, *iUpperGradient);
    else
        p.fillRect(t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4, 0,width(),18,
                   options->colorGroup(Options::TitleBar, false).
                   brush(QColorGroup::Button));
    p.setClipRegion(r);
    int i;//, ly;
/*
    for(i=0, ly=4; i < 4; ++i, ly+=3){
        p.setPen(options->color(Options::TitleBar, true).light(150));
        p.drawLine(0, ly, width()-1, ly);
        p.setPen(options->color(Options::TitleBar, true).dark(120));
        p.drawLine(0, ly+1, width()-1, ly+1);
    }
*/
    //<xxl>
    QColor base = options->color(Options::TitleBar, true);
    QColor light = base.light(130); // lichte kleur van het vierkantje
    QColor dark = base.dark(250);   //donkere kleur van het vierkantje
    //QColorGroup base = options->colorGroup(Options::TitleBar, true);
    //QColor light = base.light();
    //QColor dark = base.dark();
    int cx, cy;
    cx = 0;
    cy = 4;
//      //KwiX-title door Kristrof Borrey, e-mail:kristof.borrey@student.luc.ac.be
//      for(i=0; cx < t.x()+(((t.width()-fm.width(caption()))/2)); i++)
//      {
//          cx = i*106;
//          cy = 3;

//          //teken de strepen aan de linkerzijde
//          p.setPen(light);
//          p.drawLine(cx,cy,cx+60,cy);
//          p.drawLine(cx,cy+3,cx+60,cy+3);
//          p.drawLine(cx,cy+6,cx+60,cy+6);
//          p.drawLine(cx,cy+9,cx+60,cy+9);
//          p.setPen(dark);
//          p.drawLine(cx+1,cy+1,cx+61,cy+1);
//          p.drawLine(cx+1,cy+4,cx+61,cy+4);
//          p.drawLine(cx+1,cy+7,cx+61,cy+7);
//          p.drawLine(cx+1,cy+10,cx+61,cy+10);
//          //teken de K
//          cx = i * 106 + 64;
//          p.setPen(light);
//          p.drawLine(cx,cy,cx+2,cy);
//          p.drawLine(cx,cy,cx,cy+10);
//          p.setPen(dark);
//          p.drawLine(cx+2,cy,cx+2,cy+10);
//          p.drawLine(cx,cy+10,cx+2,cy+10);
//          //teken het tweede deel van de K
//          p.drawLine(cx+7,cy,cx+9,cy+2);
//          p.drawLine(cx+9,cy+2,cx+6,cy+5);
//          p.drawLine(cx+6,cy+5,cx+9,cy+8);
//          p.drawLine(cx+9,cy+8,cx+7,cy+10);
//          p.drawLine(cx+7,cy+10,cx+3,cy+6);
//          p.setPen(light);
//          p.drawLine(cx+3,cy+6,cx+3,cy+4);
//          p.drawLine(cx+3,cy+4,cx+7,cy);

//          //teken de W
//          cx = i * 106 + 64 + 11;
//          p.drawLine(cx,cy,cx+2,cy);
//          p.drawLine(cx+4,cy+8,cx+4,cy+5);
//          p.drawLine(cx+4,cy+5,cx+6,cy+5);
//          p.drawLine(cx+8,cy+6,cx+8,cy);
//          p.drawLine(cx+8,cy,cx+10,cy);
//          p.drawLine(cx,cy+8,cx,cy);
//          p.drawLine(cx+6,cy+8,cx+8,cy+6);
//          p.setPen(dark);
//          p.drawLine(cx+2,cy,cx+2,cy+6);
//          p.drawLine(cx+2,cy+6,cx+4,cy+8);
//          p.drawLine(cx+6,cy+5,cx+6,cy+8);
//          p.drawLine(cx+10,cy,cx+10,cy+8);
//          p.drawLine(cx+10,cy+8,cx+8,cy+10);
//          p.drawLine(cx+8,cy+10,cx+2,cy+10);
//          p.drawLine(cx+2,cy+10,cx,cy+8);

//          //teken de I
//          cx = i*106 + 64 + 11 + 12;
//          p.setPen(light);
//          p.drawLine(cx,cy+2,cx,cy);
//          p.drawLine(cx,cy,cx+2,cy);
//          p.setPen(dark);
//          p.drawLine(cx+2,cy,cx+2,cy+2);
//          p.drawLine(cx+2,cy+2,cx,cy+2);

//          p.setPen(light);
//          p.drawLine(cx,cy+10,cx,cy+3);
//          p.drawLine(cx,cy+3,cx+2,cy+3);
//          p.setPen(dark);
//          p.drawLine(cx+2,cy+3,cx+2,cy+10);
//          p.drawLine(cx+2,cy+10,cx,cy+10);

//          //teken de het linker deel van X
//          cx = i * 106 + 64 + 11 + 12 + 4;
//          p.setPen(light);
//          p.drawLine(cx,cy+8,cx+3,cy+5);
//          p.drawLine(cx,cy+2,cx+2,cy);
//          p.drawLine(cx+2,cy,cx+6,cy+4);
//          p.setPen(dark);
//          p.drawLine(cx+3,cy+5,cx,cy+2);
//          p.drawLine(cx+6,cy+4,cx+6,cy+6);
//          p.drawLine(cx+6,cy+6,cx+2,cy+10);
//          p.drawLine(cx+2,cy+10,cx,cy+8);
 //teken het rechter deel van X
 //         p.drawLine(cx+13,cy+2,cx+10,cy+5);
//          p.drawLine(cx+10,cy+5,cx+13,cy+8);
//          p.drawLine(cx+13,cy+8,cx+11,cy+10);
//          p.drawLine(cx+11,cy+10,cx+7,cy+6);
//          p.drawLine(cx+11,cy,cx+13,cy+2);
//          p.setPen(light);
//          p.drawLine(cx+7,cy+6,cx+7,cy+4);
//          p.drawLine(cx+7,cy+4,cx+11,cy);
//      cx = 10000;

       
//      }
    //    cx = 105;
    for(i=0; cx < t.x()+(((t.width()-fm.width(caption()))/2)); i++)
    {
        //teken de strepen aan de rechterzijde
        cx = cx + 2;
        p.setPen(light);
        p.drawLine(cx,cy,cx+2,cy);
        p.drawLine(cx,cy+3,cx+2,cy+3);
        p.drawLine(cx,cy+6,cx+2,cy+6);
        p.drawLine(cx,cy+9,cx+2,cy+9);
        p.setPen(dark);
        p.drawLine(cx+1,cy+1,cx+3,cy+1);
        p.drawLine(cx+1,cy+4,cx+3,cy+4);
        p.drawLine(cx+1,cy+7,cx+3,cy+7);
        p.drawLine(cx+1,cy+10,cx+3,cy+10);

    }

    cx = t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+5;
    cy = 1;
    for(i=0; i<4; i++)
    {
        cx = t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4+i*4;
        cy = i*4;
        p.fillRect(t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4,cy,4+i*4,4,base);
        p.setPen(Qt::black);
        p.drawLine(cx,cy-1,cx+3,cy-1);
        p.drawLine(cx+4,cy,cx+4,cy+3);
        p.drawPoint(cx+3,cy);
    
        p.setPen(light);
        p.drawLine(cx,cy,cx+2,cy);
        p.drawLine(cx,cy,cx,cy+2);
        p.setPen(dark);
        p.drawLine(cx+3,cy+1,cx+3,cy+3);
        p.drawLine(cx+1,cy+3,cx+3,cy+3);
    }
    p.setPen(base);
    p.drawLine(t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4,i*4,
        t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+3+i*4,i*4);
    p.setPen(light);
    p.drawLine(t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4+i*4,i*4,
        t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+6+i*4,i*4);
    p.setPen(Qt::black);
    p.drawLine(t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4+i*4,i*4-1,
        t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+7+i*4,i*4-1);
    p.drawLine(t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+7+i*4,i*4,
        t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+8+i*4,i*4);
    p.drawPoint(t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+8+i*4,i*4+1);
//            t.x()+2*((t.width()-fm.width(caption()))/2)+fm.width(caption()), 1);

    cx = t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4+13;
    cy = 3;
    p.fillRect(cx+1,cy+1,2,2,base);
    p.setPen(Qt::black);
    p.drawLine(cx-1,cy-1,cx+3,cy-1);
    p.drawLine(cx+4,cy,cx+4,cy+4);
    p.drawLine(cx,cy+4,cx+4,cy+4);
    p.drawLine(cx-1,cy-1,cx-1,cy+3);
    p.drawPoint(cx+3,cy);
    p.drawPoint(cx,cy+3);
    p.setPen(light);
    p.drawLine(cx,cy,cx+2,cy);
    p.drawLine(cx,cy,cx,cy+2);
    p.setPen(dark);
    p.drawLine(cx+3,cy+1,cx+3,cy+3);
    p.drawLine(cx+1,cy+3,cx+3,cy+3);
    cx = t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4+13+8;
    cy = 11;
    p.fillRect(cx+1,cy+1,2,2,base);
    p.setPen(Qt::black);
    p.drawLine(cx-1,cy-1,cx+3,cy-1);
    p.drawLine(cx+4,cy,cx+4,cy+4);
    p.drawLine(cx,cy+4,cx+4,cy+4);
    p.drawLine(cx-1,cy-1,cx-1,cy+3);
    p.drawPoint(cx+3,cy);
    p.drawPoint(cx,cy+3);
    p.setPen(light);
    p.drawLine(cx,cy,cx+2,cy);
    p.drawLine(cx,cy,cx,cy+2);
    p.setPen(dark);
    p.drawLine(cx+3,cy+1,cx+3,cy+3);
    p.drawLine(cx+1,cy+3,cx+3,cy+3);
    cx = t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4+13+8+8;
    cy = 14;
    p.fillRect(cx+1,cy+1,2,2,base);
    p.setPen(Qt::black);
    p.drawLine(cx-1,cy-1,cx+3,cy-1);
    p.drawLine(cx+4,cy,cx+4,cy+4);
    p.drawLine(cx,cy+4,cx+4,cy+4);
    p.drawLine(cx-1,cy-1,cx-1,cy+3);
    p.drawPoint(cx+3,cy);
    p.drawPoint(cx,cy+3);
    p.setPen(light);
    p.drawLine(cx,cy,cx+2,cy);
    p.drawLine(cx,cy,cx,cy+2);
    p.setPen(dark);
    p.drawLine(cx+3,cy+1,cx+3,cy+3);
    p.drawLine(cx+1,cy+3,cx+3,cy+3);

    //</xxl>
    p.setClipRect(t);
    p.setPen(options->color(Options::Font, true));
    p.setFont(options->font(true));

    p.drawText(t.x()+((t.width()-fm.width(caption()))/2)-4,
               0, fm.width(caption())+8, 18, AlignCenter, caption());
    p.setClipping(false);
    p.end();
    oldTitle = caption();
}

void Kwix::captionChange( const QString &)
{
    recalcTitleBuffer();
    repaint(false);
    //repaint( titlebar->geometry(), false );
}

void Kwix::iconChange(){
  if(miniIcon().isNull()) ;
     //button[5]->setIconSet(isActive() ? *menu_pix : *dis_menu_pix);
    else
        button[5]->setIconSet( miniIcon() );
    button[5]->repaint( FALSE );
}

void Kwix::drawRoundFrame(QPainter &p, int x, int y, int w, int h)
{
    kDrawRoundButton(&p, x, y, w, h,
                     options->colorGroup(Options::Frame, isActive()), false);

}

void Kwix::paintEvent( QPaintEvent* )
{
    QPainter p( this );
    QRect t = titlebar->geometry();

    QBrush fillBrush(colorGroup().brush(QColorGroup::Background).pixmap() ?
                     colorGroup().brush(QColorGroup::Background) :
                     options->colorGroup(Options::Frame, isActive()).
                     brush(QColorGroup::Button));

    p.fillRect(1, 16, width()-2, height()-16, fillBrush);
//    p.fillRect(width()-6, 0, width()-1, height(), fillBrush);

    t.setTop( 2 );
    t.setLeft( t.left() + 4 );
    t.setRight( t.right() - 2 );

    int w = width()-1; // exclude shadow
    int h = height()-1;

    // titlebar
    QColorGroup g = options->colorGroup(Options::TitleBar, isActive());
    if(isActive()){
        p.drawPixmap(1, 1, titleBuffer, 0, 0, w-2, 18);
    }
    else{
        if(iUpperGradient)
            p.drawTiledPixmap(1, 1, w-2, 18, *iUpperGradient);
        else
            p.fillRect(1, 1, w-2, 18, fillBrush);
        p.setPen(options->color(Options::Font, isActive()));
        p.setFont(options->font(isActive()));
        p.drawText(t, AlignCenter, caption() );
    }

    // titlebar highlight
    QFontMetrics fm(options->font(true));
    p.setPen(g.light());
    p.drawLine(1, 1, 1, 19);
    p.drawLine(1, 1, t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+5, 1);
    p.setPen(options->colorGroup(Options::TitleBar, false).light());
    p.drawLine(t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+5+3+2, 1, 
            t.x()+2*((t.width()-fm.width(caption()))/2)+fm.width(caption()), 1);
    p.setPen(g.dark());
    p.drawLine(w-2, 1, w-2, 19);
    int ts = t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4+5*4;
    if( t.right() < ts )
        ts = t.right();
    if( isActive() )
        p.drawLine(0, 18, ts, 18);
//    else
//        p.drawLine(0, 18, w-2, 18);
    //upper-left corner
    p.setPen(Qt::black);
    p.drawPoint(5,1);
    p.drawPoint(6,1);
    p.drawPoint(4,2);
    p.drawPoint(3,3);
    p.drawPoint(3,4);
    p.drawPoint(2,5);
    p.drawPoint(2,6);
    p.drawPoint(1,7);
    p.drawPoint(1,8);
    p.drawPoint(1,9);
    g = options->colorGroup(Options::TitleBar, isActive());
    p.setPen(g.light());
    p.drawPoint(5,2);
    p.drawPoint(6,2);
    //p.drawPoint(7,2); //
    //p.drawPoint(8,2); // border volledig linksboven   van het window
    //p.drawPoint(9,2); //
    p.drawPoint(4,3);
    p.drawPoint(4,4);
    p.drawPoint(3,5);
    p.drawPoint(3,6);
    p.drawPoint(2,7);
    p.drawPoint(2,8);
    p.drawPoint(2,9);

    // frame
    g = options->colorGroup(Options::Frame, isActive());
    p.setPen(g.light());
    p.drawLine(1, 19, 1, h-2);
    p.setPen(g.dark());
    p.drawLine(2, h-2, w-2, h-2);
    p.drawLine(w-2, 19, w-2, h-2);
    //p.drawPoint(w-3, 19);
    //p.drawPoint(2, 19);
//    qDrawShadePanel(&p, 3, 19, w-6, h-22, g, true);
 
    // skid pad
    int h1 = 18+(int)((h-18)/10);
    g = options->colorGroup(Options::TitleBar, isActive());
    p.setPen(g.light());
    p.drawLine(1,18,1,h1);
    p.setPen(g.dark());
    p.drawLine(2,18,2,h1-1);
    p.setPen(Qt::black);
    p.drawPoint(1,h1+1);
    p.drawPoint(2,h1);
    //
    int w1 = 4*(int)(w/10);
    if( isActive() )
        g = options->colorGroup(Options::TitleBar, isActive());
    else
        g = options->colorGroup(Options::Frame, isActive());
    p.setPen(g.light());
    p.drawLine(1,18,1,h-2);
    p.drawLine(1,h-3,w1,h-3);
    p.drawLine(w1+6,h-3,w1+10,h-3);
    p.drawLine(w1+14,h-3,w1+16,h-3);
    p.setPen(g.dark());
    p.drawLine(2,18,2,h-2);
    p.drawLine(1,h-2,w1,h-2);
    p.drawLine(w1+6,h-2,w1+10,h-2);
    p.drawLine(w1+14,h-2,w1+16,h-2);
    p.setPen(Qt::black);
    p.drawLine(w1,h-3,w1,h-2);
    p.drawLine(w1+5,h-3,w1+5,h-2);
    p.drawLine(w1+10,h-3,w1+10,h-2);
    p.drawLine(w1+13,h-3,w1+13,h-2);
    p.drawLine(w1+16,h-3,w1+16,h-2);
    
    p.setPen(Qt::black);
    p.drawRect(0, 0, w, h);
    p.drawRect(3,19,w-6,h-22);
    //corners
    p.setPen(Qt::black);
    p.drawPoint(1,h-17);
    p.drawPoint(2,h-16);
    p.drawPoint(16,h-2);
    p.drawPoint(15,h-3);
    g = options->colorGroup(Options::Frame, isActive());
    //p.setPen(g.light());
    p.setPen(g.dark());
    p.drawLine(1,h-16,1,h-2);
    p.drawLine(1,h-2,15,h-2);
    p.setPen(g.dark());
    p.drawLine(2,h-15,2,h-2);
    p.drawLine(1,h-3,14,h-3);
    
    p.setPen(Qt::black);
    p.drawPoint(w-15,h-3);
    p.drawPoint(w-16,h-2);
    p.drawPoint(w-3,h-16);
    p.drawPoint(w-2,h-17);
    p.setPen(g.dark());
    p.drawLine(w-14,h-3,w-2,h-3);
    p.drawLine(w-15,h-2,w-2,h-2);
    p.drawLine(w-3,h-15,w-3,h-2);
    p.drawLine(w-2,h-16,w-2,h-2);

    //shadow
    p.setPen(Qt::black);
    p.drawLine(w,2,w,h+1);
    p.drawLine(w+1,2,w+1,h+1);
    p.drawLine(2,h,w+1,h);
    p.drawLine(2,h+1,w+1,h+1);
    //p.drawLine

}

#define QCOORDARRLEN(x) sizeof(x)/(sizeof(QCOORD)*2)

void Kwix::doShape()
{
    QRegion mask;
    mask += QRect(0, 0, width()-1, height()-1);
    mask += QRect(2, 2, width()-2, height()-2);
    // upper left corner
    mask -= QRect(0,0,5,2);
    mask -= QRect(5,0,2,1);
    mask -= QRect(0,2,3,3);
    mask -= QRect(3,2,1,1);
    mask -= QRect(0,5,2,2);
    mask -= QRect(0,7,1,3);

    bool clipAroundTitle = isActive() &&
       !(oldTitle == caption() && width() == titleBuffer.width());
    //clipAroundTitle = isActive();
    clipAroundTitle = false;
    if( clipAroundTitle )
    {
        QFontMetrics fm(options->font(true));
        
        QRect t = titlebar->geometry();
        t.setTop( 2 );
        t.setLeft( t.left() + 4 );
        t.setRight( t.right() - 2 );
        
        mask -= QRect(t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+5,0,((t.width()-fm.width(caption()))/2)-3,19);
        int i,cx,cy;
        for(i=0; i<5; i++)
        {
            cx = t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4+i*4;
            cy = i*4;
        
            mask += QRect(t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4,cy,6+i*4,4);
            mask -= QRect(cx+5,cy,1,1);
        }

        cx = t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4+13;
        cy = 3;
        mask += QRect(cx,cy,5,5);
        mask += QRect(cx+1,cy+1,5,5);
        cx = t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4+13+8;
        cy = 11;
        mask += QRect(cx,cy,5,5);
        mask += QRect(cx+1,cy+1,5,5);
        cx = t.x()+((t.width()-fm.width(caption()))/2)+fm.width(caption())+4+13+8+8;
        cy = 14;
        mask += QRect(cx,cy,5,5);
        mask += QRect(cx+1,cy+1,5,5);
    }

    setMask(mask);
}

void Kwix::showEvent(QShowEvent *ev)
{
    Client::showEvent(ev);
    doShape();
    repaint();
}

void Kwix::windowWrapperShowEvent( QShowEvent* )
{
    doShape();
}

void Kwix::mouseDoubleClickEvent( QMouseEvent * e )
{
    if (titlebar->geometry().contains( e->pos() ) )
	workspace()->performWindowOperation( this, options->operationTitlebarDblClick() );
    workspace()->requestFocus( this );
}

void Kwix::stickyChange(bool on)
{
    if( button[1] )
        button[1]->setBitmap(on ? unsticky_bits : sticky_bits);
}

void Kwix::maximizeChange(bool m)
{
    if( button[3] )
        button[3]->setBitmap(m ? minmax_bits : maximize_bits);
}

void Kwix::init()
{
  Client::init();
  button[5]->setIconSet( miniIcon() );
    //
}

void Kwix::activeChange(bool /* b*/)
{
/*
    if( isActive() )
        for(int btn = 0; btn<5; btn++)
            if(button[btn])
                button[btn]->show();
    else
        for(int btn = 0; btn<5; btn++)
            if(button[btn])
                button[btn]->hide();
*/

    //doShape();
    //recalcTitleBuffer();
    //repaint(true);
    repaint(false);
    for( int i = 0; i < 6; i++ )
        if( button[i] )
            button[i]->reset();
}

Client::MousePosition Kwix::mousePosition( const QPoint& p) const
{
    MousePosition m = Client::mousePosition( p );

    if ( m == Center ) {
	int border = 10;
	if ( p.y() >= height()-border )
	    m =  Bottom;
	else if ( p.x() >= width()-border )
	    m =  Right;
    }
    return m;
}

void Kwix::createButtons(QHBoxLayout* hb)
{
    bool help = providesContextHelp();
    titleLayout[ 0]=Separator_ID;
    titleLayout[ 1]=Separator_ID;
    titleLayout[ 2]=Separator_ID;
    titleLayout[ 3]=Close_ID;
    titleLayout[ 4]=Separator_ID;
    titleLayout[ 5]=Separator_ID;
    titleLayout[ 6]=Title_ID;
    titleLayout[ 7]=Separator_ID;
    titleLayout[ 8]=Separator_ID;
    titleLayout[ 9]=Help_ID;
    titleLayout[10]=Sticky_ID;
    titleLayout[11]=Minimize_ID;
    titleLayout[12]=Maximize_ID;
    titleLayout[13]=Separator_ID;

    KConfig cfg("kwixrc");
    if( cfg.hasGroup("Buttons") )
    {
        cfg.setGroup("Buttons");
        if( cfg.hasKey("Layout") )
        {
            QStringList strl = cfg.readListEntry("Layout");
            QArray<int> tmp(strl.count());
            for( int i=0; i < (int)strl.count(); i++)
                if( strl[i] == "Separator" )
                    tmp[i] = Separator_ID;
                else if( strl[i] == "Menu" )
                    tmp[i] = Menu_ID;
                else if( strl[i] == "Title" )
                    tmp[i] = Title_ID;
                else if( strl[i] == "Help" )
                    tmp[i] = Help_ID;
                else if( strl[i] == "Sticky" )
                    tmp[i] = Sticky_ID;
                else if( strl[i] == "Minimize" )
                    tmp[i] = Minimize_ID;
                else if( strl[i] == "Maximize" )
                    tmp[i] = Maximize_ID;
                else if( strl[i] == "Close" )
                    tmp[i] = Close_ID;
                else
                    tmp[i] = Separator_ID;
            titleLayout = tmp;
        }
    }

    for( int i=0; i<6; i++ )
        button[i] = 0;

    for( int i = 0; i < (int)titleLayout.count(); i++ )
        switch( titleLayout[i] )
        {
            case Separator_ID:
                hb->addSpacing(1);
                break;
            case Menu_ID:
                button[5] = new ModernButton(this, "menu");
                button[5]->setPopupDelay(0);
                button[5]->setPopup( workspace()->clientPopup(this) );
                connect( button[5], SIGNAL( pressed() ), this, ( SLOT( menuButtonPressed() ) ) );
                hb->addWidget( button[5]);
                hb->addSpacing(1);
                break;
            case Title_ID:
                titlebar = new QSpacerItem(10, 16, QSizePolicy::Expanding,
                                           QSizePolicy::Minimum);
                hb->addItem(titlebar);
                hb->addSpacing(1);
                break;
            case Help_ID:
                if(help){
                    button[4] = new ModernButton(this, "help", question_bits);
                    connect( button[4], SIGNAL( clicked() ), this, ( SLOT( contextHelp() ) ) );
                    hb->addWidget( button[4]);
                    hb->addSpacing(1);
                }
                break;
            case Sticky_ID:
                button[1] = new ModernButton(this, "sticky");
                if(isSticky())
                    button[1]->setBitmap(unsticky_bits);
                else
                    button[1]->setBitmap(sticky_bits);
                connect( button[1], SIGNAL( clicked() ), this, ( SLOT( toggleSticky() ) ) );
                hb->addWidget( button[1]);
                hb->addSpacing(1);
                break;
            case Minimize_ID:
                button[2] = new ModernButton(this, "iconify", iconify_bits);
                connect( button[2], SIGNAL( clicked() ), this, ( SLOT( iconify() ) ) );
                hb->addWidget( button[2]);
                hb->addSpacing(1);
                break;
            case Maximize_ID:
                button[3] = new ModernButton(this, "maximize", maximize_bits);
                connect( button[3], SIGNAL( clicked() ), this, ( SLOT( maxButtonClicked() ) ) );
                hb->addWidget( button[3]);
                hb->addSpacing(1);
                break;
            case Close_ID:
                button[0] = new ModernButton(this, "close", close_bits);
                connect( button[0], SIGNAL( clicked() ), this, ( SLOT( closeWindow() ) ) );
                hb->addWidget( button[0]);
                hb->addSpacing(1);
                break;
        }
}
void Kwix::menuButtonPressed()
{
    static QTime* t = 0;
    static Kwix* tc = 0;
    if ( !t )
        t = new QTime;

    if ( tc != this || t->elapsed() > QApplication::doubleClickInterval() )
        button[5]->setPopup( workspace()->clientPopup( this ) );
    else {
        button[5]->setPopup( 0 );
        closeWindow();
    }
    t->start();
    tc = this;
}

#include "kwix.moc"
