
/*
kwix.h - header file for the "KwiX" kwin theme.

Terms and Conditions
 
Copyright (c) 2001 Catalin Climov <xxl@thekompany.com>
Parts Copyright (c) 2000-2001 Daniel M. Duley <mosfet@kde.org>
Based on the ModSystem kwin theme.
 
Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.
 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
COPYRIGHT HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
Except as contained in this notice, the name of the copyright holder shall not
be used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from the copyright holder.
*/

#ifndef __KWIX_H
#define __KWIX_H

#include <qtoolbutton.h>
#include <qbitmap.h>
#include <kpixmap.h>
//#include "../../client.h"
#include <kwin/client.h>

class QLabel;
class QSpacerItem;
class QHBoxLayout;

namespace KWinInternal {

// get rid of autohide :P
class ModernButton : public QToolButton
{
    Q_OBJECT
public:
    ModernButton(Client *parent=0, const char *name=0,
                 const unsigned char *bitmap=NULL);
    void setBitmap(const unsigned char *bitmap);
    void reset();
    QSize sizeHint() const;
protected:
    void mousePressEvent( QMouseEvent* e );
    void mouseReleaseEvent( QMouseEvent* e );

    virtual void drawButton(QPainter *p);
    void drawButtonLabel(QPainter *){;}
    QBitmap deco;
    Client *client;
public:
    int last_button;
};

class Kwix : public KWinInternal::Client
{
    Q_OBJECT
public:
    Kwix( Workspace *ws, WId w, QWidget *parent=0, const char *name=0 );
    ~Kwix(){;}
protected:
    void drawRoundFrame(QPainter &p, int x, int y, int w, int h);
    void resizeEvent( QResizeEvent* );
    void paintEvent( QPaintEvent* );
    void showEvent( QShowEvent* );
    void windowWrapperShowEvent( QShowEvent* );
    void mouseDoubleClickEvent( QMouseEvent * );
    void init();
    void captionChange( const QString& name );
  void iconChange();
    void stickyChange(bool on);
    void maximizeChange(bool m);
    void doShape();
    void recalcTitleBuffer();
    void activeChange(bool);
    MousePosition mousePosition( const QPoint& ) const;
protected slots:
    void slotReset();
    void maxButtonClicked();
    void menuButtonPressed();
private:
    enum { Separator_ID, Menu_ID, Title_ID, Help_ID, Sticky_ID, Minimize_ID, Maximize_ID, Close_ID };
    void createButtons(QHBoxLayout* hb);
    ModernButton* button[6];
    QSpacerItem* titlebar;
    QHBoxLayout* hb;
    QPixmap titleBuffer;
    QString oldTitle;
    QArray<int> titleLayout;
};

};

#endif
